%  Copyright (C) 2006 David Roundy
%
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program; see the file COPYING.  If not, write to
%  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
%  Boston, MA 02110-1301, USA.


\begin{code}
#include "gadts.h"
module Darcs.HopefullyPrivate ( -- ratify HopefullyPrivate: obvously we need it here...
                                Hopefully(..), PatchInfoAnd,
                                piap, patchInfoAndPatch,
                                conscientiously, hopefully, info,
                                hopefullyM, createHashed, extractHash,
                                actually, unavailable ) where

import System.IO.Unsafe ( unsafeInterleaveIO )
import Control.Monad ( liftM )

import Printer ( Doc, renderString, errorDoc, text, ($$) )
import Darcs.Patch.Info ( PatchInfo, human_friendly, idpatchinfo )
import Darcs.Patch ( Named )
import Darcs.Patch.Effect ( Effect )
import Darcs.Patch.Patchy ( Invert(..), ShowPatch(..), Commute(..) )
import Darcs.Patch.Ordered ( MyEq, unsafeCompare, (:>)(..), (:\/:)(..), (:/\:)(..) )

data Hopefully a = Hopefully (SimpleHopefully a) | Hashed String (SimpleHopefully a)

data SimpleHopefully a = Actually a | Unavailable String

data PatchInfoAnd p C(a b) = PIAP !PatchInfo (Hopefully (Named p C(a b)))

instance Monad Hopefully where
    return a = Hopefully (Actually a)
    (Hopefully (Unavailable e)) >>= _ = Hopefully (Unavailable e)
    (Hashed h (Unavailable e)) >>= _ = Hashed h (Unavailable e)
    (Hopefully (Actually a)) >>= f = f a
    (Hashed _ (Actually a)) >>= f = f a
    fail e = Hopefully (Unavailable e)

instance Functor Hopefully where
    fmap = liftM

info :: PatchInfoAnd p C(a b) -> PatchInfo
info (PIAP i _) = i

piap :: PatchInfo -> Named p C(a b) -> PatchInfoAnd p C(a b)
piap i p = PIAP i (Hopefully $ Actually p)

patchInfoAndPatch :: PatchInfo -> Hopefully (Named p C(a b)) -> PatchInfoAnd p C(a b)
patchInfoAndPatch =  PIAP

hopefully :: PatchInfoAnd p C(a b) -> Named p C(a b)
hopefully = conscientiously $ \e -> text "failed to read patch:" $$ e

conscientiously :: (Doc -> Doc)
                -> PatchInfoAnd p C(a b) -> Named p C(a b)
conscientiously er (PIAP pinf hp) =
    case hopefully2either hp of
      Right p -> p
      Left e -> errorDoc $ er (human_friendly pinf $$ text e)

hopefullyM :: Monad m => PatchInfoAnd p C(a b) -> m (Named p C(a b))
hopefullyM (PIAP pinf hp) = case hopefully2either hp of
                              Right p -> return p
                              Left e -> fail $ renderString (human_friendly pinf $$ text e)

-- Any recommendations for a nice adverb to name the below?
hopefully2either :: Hopefully a -> Either String a
hopefully2either (Hopefully (Actually p)) = Right p
hopefully2either (Hashed _ (Actually p)) = Right p
hopefully2either (Hopefully (Unavailable e)) = Left e
hopefully2either (Hashed _ (Unavailable e)) = Left e

actually :: a -> Hopefully a
actually = Hopefully . Actually

createHashed :: String -> (String -> IO a) -> IO (Hopefully a)
createHashed h f = do x <- unsafeInterleaveIO (fmap Actually (f h) `catch`
                                               \e -> return (Unavailable $ show e))
                      return (Hashed h x)

extractHash :: PatchInfoAnd p C(a b) -> Either (Named p C(a b)) String
extractHash (PIAP _ (Hashed s _)) = Right s
extractHash hp = Left $ conscientiously (\e -> text "unable to read patch:" $$ e) hp

unavailable :: String -> Hopefully a
unavailable = Hopefully . Unavailable

instance MyEq p => MyEq (PatchInfoAnd p) where
    unsafeCompare (PIAP i _) (PIAP i2 _) = i == i2

instance Invert p => Invert (PatchInfoAnd p) where
    identity = PIAP idpatchinfo (actually identity)
    invert (PIAP i p) = PIAP i (invert `fmap` p)

instance (Effect p, ShowPatch p) => ShowPatch (PatchInfoAnd p) where
    showPatch (PIAP n p) = case hopefully2either p of
                           Right x -> showPatch x
                           Left _ -> human_friendly n
    showContextPatch s (PIAP n p) = case hopefully2either p of
                                    Right x -> showContextPatch s x
                                    Left _ -> human_friendly n
    description (PIAP n _) = human_friendly n
    summary (PIAP n p) = case hopefully2either p of
                         Right x -> summary x
                         Left _ -> human_friendly n
    showNicely (PIAP n p) = case hopefully2either p of
                            Right x -> showNicely x
                            Left _ -> human_friendly n

instance Commute p => Commute (PatchInfoAnd p) where
    commute (x :> y) = do y' :> x' <- commute (hopefully x :> hopefully y)
                          return $ (info y `piap` y') :> (info x `piap` x')
    list_touched_files = list_touched_files . hopefully
    merge (x :\/: y) = case merge (hopefully x :\/: hopefully y) of
                       y' :/\: x' -> (info y `piap` y') :/\: (info x `piap` x')
\end{code}

